import mugrade
import numpy as np
import pandas as pd

def test_load_data():
    data_frame = load_data()
    with mugrade.test:
        assert (lambda df : {col:str(df[col].dtype) for col in df.columns})(data_frame) == {'vid': 'int64', 'tmstmp': 'datetime64[ns]', 'lat': 'float64', 'lon': 'float64', 'hdg': 'int64', 'pid': 'int64', 'rt': 'object', 'des': 'object', 'pdist': 'int64', 'spd': 'int64', 'tablockid': 'object', 'tatripid': 'int64'}
    with mugrade.test:
        assert len(data_frame) == 215473
    with mugrade.test:
        assert isinstance(data_frame.index, pd.DatetimeIndex) == True

def submit_load_data():
    data_frame = load_data()
    mugrade.submit((data_frame["tmstmp"] > pd.to_datetime("2016-08-15")).sum())
    mugrade.submit((data_frame["des"] == "Swissvale").sum())

def test_split_vehicles():
    vehicles = split_vehicles(load_data())
    with mugrade.test:
        assert len(vehicles) == 180
    with mugrade.test:
        assert all(isinstance(v, pd.DataFrame) for v in vehicles) == True
    with mugrade.test:
        assert all(v._is_copy for v in vehicles) == True

def submit_split_vehicles():
    vehicles = split_vehicles(load_data())
    mugrade.submit(sum(len(v) for v in vehicles))
    mugrade.submit(len(sorted(vehicles, key = lambda x : len(x), reverse=True)[0]))

def test_get_speeds():
    data = load_data()
    with mugrade.test:
        assert list(get_speeds(data, "10:00am", "10:30am").index.names) == ['vid']
    with mugrade.test:
        assert len(get_speeds(data, "10:00am", "10:30am")) == 95
    with mugrade.test:
        assert round(get_speeds(data, "10:00am", "10:30am").loc[3200],2) == 18.22

def submit_get_speeds():
    data = load_data()
    mugrade.submit(len(get_speeds(data, "9:00am", "9:30am")))
    mugrade.submit(len(get_speeds(data, "12:00am", "12:30am")))
    mugrade.submit(round(get_speeds(data, "12:00am", "12:30am").loc[5436],2))

def test_visualize_speeds():
    img_plot = visualize_speeds(load_data(), pd.period_range(start='12:00am', end='11:59pm', freq="15T"), list(range(0, 50, 2)))
    with mugrade.test:
        assert img_plot.get_array().shape == (24, 95)
    with mugrade.test:
        assert np.all(img_plot.get_array().data[10:,50:60] == [
            [6.0, 11.0, 9.0, 10.0, 14.0, 15.0, 17.0, 17.0, 20.0, 17.0], 
            [13.0, 10.0, 9.0, 6.0, 7.0, 17.0, 7.0, 12.0, 9.0, 11.0], 
            [2.0, 6.0, 6.0, 4.0, 6.0, 4.0, 3.0, 7.0, 5.0, 4.0], 
            [4.0, 3.0, 1.0, 2.0, 1.0, 1.0, 8.0, 0.0, 2.0, 3.0], 
            [1.0, 2.0, 2.0, 3.0, 1.0, 1.0, 6.0, 0.0, 3.0, 1.0], 
            [1.0, 0.0, 0.0, 1.0, 0.0, 2.0, 1.0, 2.0, 0.0, 1.0], 
            [0.0, 1.0, 0.0, 1.0, 0.0, 1.0, 0.0, 2.0, 0.0, 0.0], 
            [1.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0], 
            [1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0], 
            [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 0.0, 1.0], 
            [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], 
            [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], 
            [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], 
            [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0]])

def submit_visualize_speeds():
    img_plot = visualize_speeds(load_data(), pd.period_range(start='12:00am', end='11:59pm', freq="30T"), list(range(0, 50, 2)))
    mugrade.submit(img_plot.get_array().shape)
    mugrade.submit(img_plot.get_array().data[10:,25:35])

def test_closest_pdist():
    series = closest_pdist(load_data())
    with mugrade.test:
        assert len(series) == 7601
    with mugrade.test:
        assert len(series.loc[4663]) == 897
    with mugrade.test:
        assert np.allclose(series.loc[4663].values[:20], [25257.0, 22125.5, 12444.0, 16886.0, 14306.8, 17765.5, 13373.0, 4417.699999999997, 12700.7, 7511.0, 16915.4, 8650.699999999997, 13789.600000000006, 4522.300000000003, 15568.466666666667, 20697.199999999997, 14796.100000000006, 40383.1, 15716.5, 20405.95], atol=1e-8)

def submit_closest_pdist():
    series = closest_pdist(load_data())
    mugrade.submit(len(series.loc[4521]))
    mugrade.submit(series.loc[4521].values[:20])
